@@ -1,22 +1,69 @@
 """Mysql Database"""
 from tkinter import ttk,Tk,Label,Entry
-from shaonutil.strings import generateCryptographicallySecureRandomString
 import tkinter as tk
+from shaonutil.strings import generateCryptographicallySecureRandomString
 import mysql.connector as mysql
-
+import subprocess
+import shaonutil
+import os
 
 class MySQL:
 	"""A class for all mysql actions"""
-	def __init__(self,config,log=False):
+	def __init__(self,config,init_start_server=True,log=False):
 		self.log = log
+		self.init_start_server = init_start_server
 		self.config = config
 
+	def start_mysql_server(self):
+		"""Start mysql server"""
+		# --defaults-file=my.ini ->  mysql_config_file
+		# instead
+		# --port
+		
+		# --console If you omit the --console option, the server writes diagnostic output to the error log in the data directory
+		# --log-error = error file
+		# --debug - mysqld writes a log file C:\mysqld.trace that should contain the reason why mysqld doesn't start
+
+		mysql_bin_folder = self._config['mysql_bin_folder']
+
+		DETACHED_PROCESS = 0x00000008
+		pids = shaonutil.process.is_process_exist('mysqld.exe')
+		if not pids:
+			#process = subprocess.Popen([os.path.join(mysql_bin_folder,"mysqld.exe"),"--defaults-file="+mysql_config_file,"--standalone","--debug"],creationflags=DETACHED_PROCESS)
+			process = subprocess.Popen([os.path.join(mysql_bin_folder,"mysqld.exe"),"--standalone","--debug"],creationflags=DETACHED_PROCESS)
+			print("Starting mysql server at pid",process.pid)
+			return process
+		else:
+			print("MYSQL Server is already running at pids",pids)
+
+	def stop_mysql_server(self,force=False):
+		"""Stop MySQL Server"""
+		# mysql_config_file
+		mysql_bin_folder = self._config['mysql_bin_folder']
+		user = self._config['user']
+		password = self._config['password']
+
+		pids = shaonutil.process.is_process_exist('mysqld.exe')
+		if pids:
+			print("MYSQL running at pids",pids)
+			if force == True:
+				shaonutil.process.killProcess_ByAll("mysqld.exe")
+				print("Forced stop MySQL Server ...")
+			else:
+				DETACHED_PROCESS = 0x00000008	
+				#mysqladmin -u robist_shaon --password=sh170892  shutdown
+				process = subprocess.Popen([os.path.join(mysql_bin_folder,"mysqladmin.exe"),"-u",user,"--password="+password,"shutdown"]) # ,creationflags=DETACHED_PROCESS
+				print("Stopping MySQL Server ...")
+		else:
+			print("MYSQL Server is not already running... , you can not close.")
+
 	def reopen_connection(self):
 		"""reopen"""
 		print("MySQL > Explicitly opening connection ...")
 		self.make_cursor()
 
 	def close_connection(self):
+		"""closing the connection"""
 		print("MySQL > Explicitly closing connection ...")
 		self._cursor.close()
 		self.mySQLConnection.close()
@@ -39,6 +86,9 @@ class MySQL:
 				break
 
 	def make_cursor(self):
+		if self.init_start_server:
+			self.start_mysql_server()
+
 		try:
 			# Connection parameters and access credentials
 			if 'database' in self._config:
@@ -63,6 +113,7 @@ class MySQL:
 		self._cursor = mySQLConnection.cursor()
 
 	def is_mysql_user_exist(self,mysql_username):
+		"""check if mysql user exist return type:boolean"""
 		mySQLCursor = self._cursor
 		mySqlListUsers = "select host, user from mysql.user;"
 		mySQLCursor.execute(mySqlListUsers)
@@ -89,6 +140,7 @@ class MySQL:
 	               querynum=0, 
 	               updatenum=0, 
 	               connection_num=0):
+		"""Create a Mysql User"""
 		cursor = self._cursor
 		try:
 			print("MySQL > Creating user",userName)
@@ -101,6 +153,7 @@ class MySQL:
 	               querynum=0, 
 	               updatenum=0, 
 	               connection_num=0):
+		"""Grant a user all privilages"""
 		cursor = self._cursor
 		try:
 			print("MySQL > Granting all PRIVILEGES to user",userName)
@@ -112,6 +165,7 @@ class MySQL:
 			print("Error creating MySQL User: %s"%(Ex));
 
 	def is_db_exist(self,dbname):
+		"""Check if database exist"""
 		cursor = self._cursor
 
 		cursor.execute("SHOW DATABASES")
@@ -123,11 +177,13 @@ class MySQL:
 			return True
 
 	def create_db(self,dbname):
+		"""Create Database"""
 		cursor = self._cursor
 		print("MySQL > Creating database "+dbname+" ...")
 		cursor.execute("CREATE DATABASE "+dbname)
 
 	def is_table_exist(self,tbname):
+		"""Check if table exist"""
 		cursor = self._cursor
 		cursor.execute('SHOW TABLES')
 		tables = cursor.fetchall()
@@ -138,6 +194,7 @@ class MySQL:
 			return False
 
 	def create_table(self,tbname,column_info):
+		"""Create a table under a database"""
 		cursor = self._cursor
 		print("MySQL > Creating table "+tbname+" ...")
 		cursor.execute("CREATE TABLE "+tbname+" (id INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,"+''.join([' '+info+' '+column_info[info]+',' for info in column_info])[:-1]+")")
@@ -150,19 +207,21 @@ class MySQL:
 		# [('id', 'int(11)', 'NO', 'PRI', None, 'auto_increment'), ('book_sid', 'varchar(255)', 'YES', '', None, ''), ('book_name', 'varchar(255)', 'YES', '', None, ''), ('book_writer', 'varchar(255)', 'YES', '', None, ''), ('book_publisher', 'varchar(255)', 'YES', '', None, '')]
 
 	def get_columns_names(self,tbname):
+		"""Get column names of a table"""
 		columns = self.get_columns(tbname)
 		return [column_name for column_name, *_ in columns]
 
-	def get_unique_id_from_field(self,key_length,field_name):
+	def get_unique_id_from_field(self,field_name,key_length,filters=[]):
+		"""Get a random unique id not registered in a specific field"""
 		table = self._config['table']
 
 		cursor = self._cursor
 
-		sid = generateCryptographicallySecureRandomString(key_length)
+		sid = generateCryptographicallySecureRandomString(stringLength=key_length,filters=filters)
+		
 		
 		while True:
 			query = "SELECT * FROM "+table+" WHERE `"+field_name+"` = '"+sid+"'"
-			print(query)
 
 			## getting records from the table
 			cursor.execute(query)
@@ -177,7 +236,7 @@ class MySQL:
 
 			if(len(records)>1):
 				print("matched with previously stored sid")
-				sid = generateCryptographicallySecureRandomString(key_length)
+				sid = generateCryptographicallySecureRandomString(stringLength=key_length,filters=filters)
 			else:
 				print("Got unique sid")
 				break
@@ -185,6 +244,7 @@ class MySQL:
 		return sid
 
 	def insert_data(self,value_tupple):
+		"""Insert row of data"""
 		cursor = self._cursor
 		dbname = self._config['database']
 		tbname = self._config['table']
@@ -208,7 +268,7 @@ class MySQL:
 
 		print(cursor.rowcount, "record inserted")
 
-def create_configuration(option='cli'):
+def create_configuration(option='cli',file_name = "private/config.ini"):
 	"""Creating Configuration"""
 	if option == 'cli':
 		print('Getting your configurations to save it.\n')
@@ -221,9 +281,24 @@ def create_configuration(option='cli'):
 
 		mysql_bin_folder = input('Give your path of mysql bin folder : ')
 
-		f = open("private/config.ini", "w+")
-		f.writelines(["; config file\n", "[db_authentication]\n", "host = "+dbhost+"\n", "user = "+dbuser+"\n", "password = "+dbpassword+"\n", "database = "+dbname+"\n", "table = "+dbtable+"\n","[MYSQL]\n","mysql_bin_folder = "+mysql_bin_folder+"\n","[DB_INITIALIZE]\n","host = localhost\n","usr = root\n","passwd = \n"])
-		f.close()
+		configstr = f"""; config file
+[DB_INITIALIZE]
+mysql_bin_folder = {mysql_bin_folder}
+host = localhost
+user = root
+password = 
+[DB_AUTHENTICATION]
+mysql_bin_folder = {mysql_bin_folder}
+host = {dbhost}
+user = {dbuser}
+password = {dbpassword}
+database = {dbname}
+table = {dbtable}
+[MYSQL]
+mysql_bin_folder = {mysql_bin_folder}"""
+
+		shaonutil.file.write_file(file_name,configstr)
+
 	elif option == 'gui':
 		window = Tk()
 		window.title("Welcome to DB Config")
@@ -266,12 +341,45 @@ def create_configuration(option='cli'):
 			dbname = dbname_.get()
 			dbtable = dbtable_.get()
 
-			f = open("private/config.ini", "w+")
-			f.writelines(["; config file\n", "[db_authentication]\n", "host = "+dbhost+"\n", "user = "+dbuser+"\n", "password = "+dbpassword+"\n", "database = "+dbname+"\n", "table = "+dbtable+"\n","[MYSQL]\n","mysql_bin_folder = "+mysql_bin_folder+"\n","[DB_INITIALIZE]\n","host = localhost\n","usr = root\n","passwd = \n"])
-			f.close()
+			configstr = f"""; config file
+[DB_INITIALIZE]
+mysql_bin_folder = {mysql_bin_folder}
+host = localhost
+user = root
+password = 
+[DB_AUTHENTICATION]
+mysql_bin_folder = {mysql_bin_folder}
+host = {dbhost}
+user = {dbuser}
+password = {dbpassword}
+database = {dbname}
+table = {dbtable}
+[MYSQL]
+mysql_bin_folder = {mysql_bin_folder}"""
+
+			shaonutil.file.write_file(file_name,configstr)
+
 			window.destroy()
 
 
 
+
 		btn = ttk.Button(window ,text="Submit",command=clicked).grid(row=9,column=0)
-		window.mainloop()
\ No newline at end of file
+		window.mainloop()
+
+
+def remove_aria_log(mysql_data_dir):
+	"""Removing aria_log.### files to in mysql data dir to restart mysql"""
+	aria_log_files = [file for file in os.listdir(mysql_data_dir) if 'aria_log.' in file]
+
+	for aria_log in aria_log_files:
+		aria_log = os.path.join(mysql_data_dir,aria_log)
+		os.remove(aria_log)
+
+def get_mysql_datadir(mysql_bin_folder,user,pass_=''):
+	"""Get mysql data directory"""
+	process = subprocess.Popen([os.path.join(mysql_bin_folder,"mysql"),"--user="+user,"--password="+pass_,"-e","select @@datadir;"],stdout=subprocess.PIPE)
+	out, err = process.communicate()
+	out = [line for line in out.decode('utf8').replace("\r\n","\n").split('\n') if line != ''][-1]
+	datadir = out.replace('\\\\','\\')
+	return datadir
\ No newline at end of file
